@import 'colors';

// incognito mode

$incognito_color: if($variant == 'light', #d7e3f0, #1c2438);
$incognito_icon: 'user-not-tracked-symbolic';

// automation mode
$automation_color: #ff9600;
$automation_edge_color: lighten($automation_color, 13%);

// utility function/macros and imports from the base GTK+ theme.

@function themecolor($s) {
  @return unquote("@" + "#{$s}");
}

@function gtkalpha($c,$a) {
  @return unquote("alpha(#{$c},#{$a})");
}
