/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * Alternatively, the contents of this file may be used under the terms
 * of the GNU General Public License Version 3, as described below:
 *
 * This file is free software: you may copy, redistribute and/or modify
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * This file is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see http://www.gnu.org/licenses/.
 *
 * Changes
 * - replace Firefox specific css extensions with WebKit css extensions
 * - append FeedView css selectors
 *
 * - Adjusted for Epiphany (Removing footer)
 *
 * Notes:
 *
 * - WCAG 2.0 level AA recommends at least 4.5 for normal text, 3 for large
 *   text. See: https://marijohannessen.github.io/color-contrast-checker/
 *
 * - The @font-face rules try to use locally installed copies of the fonts,
 *   and fallback to the bundled versions. As per W3C recommendation, the
 *   rules include both full font names and PostScript names to assure
 *   proper matching across platforms and installed font formats. See:
 *   https://www.w3.org/TR/css-fonts-3/#font-face-rule
 */


body.sans {
  font-family: sans-serif;
}

body.serif {
  font-family: serif;
}

body {
  line-height: 1.45;
  text-rendering: optimizeLegibility;
}

body.light {
  /* Contrast ratio: 10.49 */
  color: #333333;
  background: #efefee;
}

body.dark {
  color: #efefef;
  background: #181818;
}

h1,
h2,
h3,
h4,
h5,
h6 {
  /* Contrast ratio: 4.77 */
  font-weight: bold;
  color: #555555;
  font-family: serif;
}

body.serif h1,
body.serif h2,
body.serif h3,
body.serif h4,
body.serif h5,
body.serif h6 {
  font-family: sans-serif;
}


body.dark h1,
body.dark h2,
body.dark h3,
body.dark h4,
body.dark h5,
body.dark h6 {
  color: #999999;
}


tt,
code,
pre {
  font-family: ephy-reader-mono, monospace;
}

hr {
  border: 1px solid #efefef;
}

picture img {
  border-radius: 3px;
}

h1 {
  font-size: 1.6em;
}

h2 {
  font-size: 1.2em;
}

h3 {
  font-size: 1em;
}

a {
  text-decoration: underline;
  font-weight: normal;
}

a,
a:visited,
a:hover,
a:active {
  color: #0095dd;
}

* {
  max-width: 100%;
  height: auto;
}

p,
code,
pre,
blockquote,
ul,
ol,
li,
figure,
.wp-caption {
  margin: 0 0 30px 0;
}

p > img:only-child,
p > a:only-child > img:only-child,
.wp-caption img,
figure img {
  display: block;
}

.caption,
.wp-caption-text,
figcaption {
  font-size: 0.9em;
  font-style: italic;
}

code,
pre {
  white-space: pre-wrap;
}

blockquote {
  padding: 0;
  -webkit-padding-start: 16px;
}

ul,
ol {
  padding: 0;
}

ul {
  -webkit-padding-start: 30px;
  list-style: disc;
}

ol {
  -webkit-padding-start: 30px;
  list-style: decimal;
}

/* Hide elements with common "hidden" class names */
.visually-hidden,
.visuallyhidden,
.hidden,
.invisible,
.sr-only {
  display: none;
}

article {
  overflow-y: hidden;
  margin: 20px auto;
  width: 640px;
  font-size: 18px;
  word-wrap: break-word;
}

mark {
  padding: 2px;
  margin: -2px;
  border-radius: 3px;
  background: #d7d7d8;
}

body.dark mark {
  background: #282828;
  color: #efefef;
}
